@@ -48,6 +48,7 @@
 #include "src/util/pmix_os_path.h"
 #include "src/util/pmix_path.h"
 #include "src/util/pmix_environ.h"
+#include "src/util/proc_info.h"
 #include "src/util/prte_cmd_line.h"
 #include "src/runtime/pmix_init_util.h"
 #include "src/util/session_dir.h"
@@ -403,12 +404,27 @@ cleanup:
     return rc;
 }
 
+static bool mcaoption(char *s)
+{
+    size_t len = strlen(s);
+
+    if (3 > len) {
+        return false;
+    }
+    if ('a' == s[len-1] &&
+        'c' == s[len-2] &&
+        'm' == s[len-3]) {
+        return true;
+    }
+    return false;
+}
+
 static int parse_cli(char **argv, pmix_cli_result_t *results,
                      bool silent)
 {
-    int rc, n;
+    int rc, m, n;
     pmix_cli_item_t *opt;
-    char *p1;
+    char *p1, *p2;
     char **pargv;
 
     /* backup the argv */
@@ -435,10 +451,27 @@ static int parse_cli(char **argv, pmix_cli_result_t *results,
 
     /* Convert single dashes to multi-dashes. */
     for (n=1; NULL != pargv[n]; n++) {
+        if (0 == strcmp("--", pargv[n])) {
+            // quit processing
+            break;
+        }
         /* check for option */
         if ('-' != pargv[n][0]) {
             continue;
         }
+        // if this is an mca spec, then skip it
+        if (mcaoption(pargv[n])) {
+            if ('-' != pargv[n][1]) {
+                // make it a "--" option
+                p2 = strdup(pargv[n]);
+                free(pargv[n]);
+                pmix_asprintf(&pargv[n], "-%s", p2);
+                free(p2);
+            }
+            // now skip the next two positions
+            n += 2;
+            continue;
+        }
         /* check for single-dash errors */
         if ('-' != pargv[n][1] && 2 < strlen(pargv[n])) {
             /* we know this is incorrect */
@@ -625,7 +658,7 @@ static int convert_deprecated_cli(pmix_cli_result_t *results,
                                                 warn);
             PMIX_CLI_REMOVE_DEPRECATED(results, opt);
         }
-        /* --use-hwthread-cpus -> --bind-to hwthread */
+        /* --use-hwthread-cpus -> --map-by hwtcpus */
         else if (0 == strcmp(option, "use-hwthread-cpus")) {
             rc = prte_schizo_base_add_qualifier(results, option,
                                                 PRTE_CLI_MAPBY, PRTE_CLI_HWTCPUS,
@@ -1763,6 +1796,17 @@ static int parse_env(char **srcenv, char ***dstenv,
         PMIX_ARGV_FREE_COMPAT(xvals);
     }
 
+#if PMIX_NUMERIC_VERSION != 0x00040208
+    /* add a flag indicating that we did indeed check the tmpdir
+     * for a shared file system */
+    if (prte_process_info.shared_fs) {
+        p1 = "TRUE";
+    } else {
+        p1 = "FALSE";
+    }
+    PMIX_SETENV_COMPAT("PRTE_SHARED_FS", p1, true, dstenv);
+#endif
+
     return PRTE_SUCCESS;
 }
 
@@ -1801,7 +1845,23 @@ static bool check_prte_overlap(char *var, char *value)
         return true;
     } else if (0 == strncmp(var, "reachable_", strlen("reachable_"))) {
         // need to convert reachable to prtereachable
-        pmix_asprintf(&tmp, "PRTE_MCA_prtereachable_%s", &var[strlen("reachable")]);
+        pmix_asprintf(&tmp, "PRTE_MCA_prtereachable_%s", &var[strlen("reachable_")]);
+        // set it, but don't overwrite if they already
+        // have a value in our environment
+        setenv(tmp, value, false);
+        free(tmp);
+        return true;
+    } else if (0 == strncmp(var, "plm_rsh_", strlen("plm_rsh_"))) {
+        // need to convert rsh to ssh
+        pmix_asprintf(&tmp, "PRTE_MCA_plm_ssh_%s", &var[strlen("plm_rsh_")]);
+        // set it, but don't overwrite if they already
+        // have a value in our environment
+        setenv(tmp, value, false);
+        free(tmp);
+        return true;
+    } else if (0 == strncmp(var, "orte_", strlen("orte_"))) {
+        // need to convert "orte" to "prte"
+        pmix_asprintf(&tmp, "PRTE_MCA_prte_%s", &var[strlen("orte_")]);
         // set it, but don't overwrite if they already
         // have a value in our environment
         setenv(tmp, value, false);
