@@ -5,7 +5,7 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2020      IBM Corporation.  All rights reserved.
  * Copyright (c) 2020      Cisco Systems, Inc.  All rights reserved
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -65,7 +65,7 @@ void prte_state_base_activate_job_state(prte_job_t *jdata, prte_job_state_t stat
             error = itm;
         }
         if (s->job_state == state) {
-            PRTE_REACHING_JOB_STATE(jdata, state, s->priority);
+            PRTE_REACHING_JOB_STATE(jdata, state);
             if (NULL == s->cbfunc) {
                 PMIX_OUTPUT_VERBOSE((1, prte_state_base_framework.framework_output,
                                      "%s NULL CBFUNC FOR JOB %s STATE %s",
@@ -80,7 +80,7 @@ void prte_state_base_activate_job_state(prte_job_t *jdata, prte_job_state_t stat
                 caddy->job_state = state;
                 PMIX_RETAIN(jdata);
             }
-            PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc, s->priority);
+            PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc);
             return;
         }
     }
@@ -108,11 +108,11 @@ void prte_state_base_activate_job_state(prte_job_t *jdata, prte_job_state_t stat
         caddy->job_state = state;
         PMIX_RETAIN(jdata);
     }
-    PRTE_REACHING_JOB_STATE(jdata, state, s->priority);
-    PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc, s->priority);
+    PRTE_REACHING_JOB_STATE(jdata, state);
+    PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc);
 }
 
-int prte_state_base_add_job_state(prte_job_state_t state, prte_state_cbfunc_t cbfunc, int priority)
+int prte_state_base_add_job_state(prte_job_state_t state, prte_state_cbfunc_t cbfunc)
 {
     pmix_list_item_t *item;
     prte_state_t *st;
@@ -131,7 +131,6 @@ int prte_state_base_add_job_state(prte_job_state_t state, prte_state_cbfunc_t cb
     st = PMIX_NEW(prte_state_t);
     st->job_state = state;
     st->cbfunc = cbfunc;
-    st->priority = priority;
     pmix_list_append(&prte_job_states, &(st->super));
 
     return PRTE_SUCCESS;
@@ -155,28 +154,11 @@ int prte_state_base_set_job_state_callback(prte_job_state_t state, prte_state_cb
     st = PMIX_NEW(prte_state_t);
     st->job_state = state;
     st->cbfunc = cbfunc;
-    st->priority = PRTE_SYS_PRI;
     pmix_list_append(&prte_job_states, &(st->super));
 
     return PRTE_SUCCESS;
 }
 
-int prte_state_base_set_job_state_priority(prte_job_state_t state, int priority)
-{
-    pmix_list_item_t *item;
-    prte_state_t *st;
-
-    for (item = pmix_list_get_first(&prte_job_states); item != pmix_list_get_end(&prte_job_states);
-         item = pmix_list_get_next(item)) {
-        st = (prte_state_t *) item;
-        if (st->job_state == state) {
-            st->priority = priority;
-            return PRTE_SUCCESS;
-        }
-    }
-    return PRTE_ERR_NOT_FOUND;
-}
-
 int prte_state_base_remove_job_state(prte_job_state_t state)
 {
     pmix_list_item_t *item;
@@ -226,7 +208,7 @@ void prte_state_base_activate_proc_state(pmix_proc_t *proc, prte_proc_state_t st
             error = itm;
         }
         if (s->proc_state == state) {
-            PRTE_REACHING_PROC_STATE(proc, state, s->priority);
+            PRTE_REACHING_PROC_STATE(proc, state);
             if (NULL == s->cbfunc) {
                 PMIX_OUTPUT_VERBOSE((1, prte_state_base_framework.framework_output,
                                      "%s NULL CBFUNC FOR PROC %s STATE %s",
@@ -237,7 +219,7 @@ void prte_state_base_activate_proc_state(pmix_proc_t *proc, prte_proc_state_t st
             caddy = PMIX_NEW(prte_state_caddy_t);
             caddy->name = *proc;
             caddy->proc_state = state;
-            PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc, s->priority);
+            PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc);
             return;
         }
     }
@@ -249,8 +231,8 @@ void prte_state_base_activate_proc_state(pmix_proc_t *proc, prte_proc_state_t st
     } else if (NULL != any) {
         s = (prte_state_t *) any;
     } else {
-        PMIX_OUTPUT_VERBOSE(
-            (1, prte_state_base_framework.framework_output, "INCREMENT: ANY STATE NOT FOUND"));
+        PMIX_OUTPUT_VERBOSE((1, prte_state_base_framework.framework_output,
+                             "INCREMENT: ANY STATE NOT FOUND"));
         return;
     }
     if (NULL == s->cbfunc) {
@@ -261,12 +243,11 @@ void prte_state_base_activate_proc_state(pmix_proc_t *proc, prte_proc_state_t st
     caddy = PMIX_NEW(prte_state_caddy_t);
     caddy->name = *proc;
     caddy->proc_state = state;
-    PRTE_REACHING_PROC_STATE(proc, state, s->priority);
-    PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc, s->priority);
+    PRTE_REACHING_PROC_STATE(proc, state);
+    PRTE_PMIX_THREADSHIFT(caddy, prte_event_base, s->cbfunc);
 }
 
-int prte_state_base_add_proc_state(prte_proc_state_t state, prte_state_cbfunc_t cbfunc,
-                                   int priority)
+int prte_state_base_add_proc_state(prte_proc_state_t state, prte_state_cbfunc_t cbfunc)
 {
     pmix_list_item_t *item;
     prte_state_t *st;
@@ -285,7 +266,6 @@ int prte_state_base_add_proc_state(prte_proc_state_t state, prte_state_cbfunc_t
     st = PMIX_NEW(prte_state_t);
     st->proc_state = state;
     st->cbfunc = cbfunc;
-    st->priority = priority;
     pmix_list_append(&prte_proc_states, &(st->super));
 
     return PRTE_SUCCESS;
@@ -307,22 +287,6 @@ int prte_state_base_set_proc_state_callback(prte_proc_state_t state, prte_state_
     return PRTE_ERR_NOT_FOUND;
 }
 
-int prte_state_base_set_proc_state_priority(prte_proc_state_t state, int priority)
-{
-    pmix_list_item_t *item;
-    prte_state_t *st;
-
-    for (item = pmix_list_get_first(&prte_proc_states);
-         item != pmix_list_get_end(&prte_proc_states); item = pmix_list_get_next(item)) {
-        st = (prte_state_t *) item;
-        if (st->proc_state == state) {
-            st->priority = priority;
-            return PRTE_SUCCESS;
-        }
-    }
-    return PRTE_ERR_NOT_FOUND;
-}
-
 int prte_state_base_remove_proc_state(prte_proc_state_t state)
 {
     pmix_list_item_t *item;
@@ -580,12 +544,6 @@ void prte_state_base_track_procs(int fd, short argc, void *cbdata)
             PMIx_server_deregister_client(proc, opcbfunc, &lock);
             PRTE_PMIX_WAIT_THREAD(&lock);
             PRTE_PMIX_DESTRUCT_LOCK(&lock);
-
-            /* Clean up the session directory as if we were the process
-             * itself.  This covers the case where the process died abnormally
-             * and didn't cleanup its own session directory.
-             */
-            prte_session_dir_finalize(proc);
         }
         /* if we are trying to terminate and our routes are
          * gone, then terminate ourselves IF no local procs
