@@ -4,8 +4,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2020-2021 IBM Corporation.  All rights reserved.
  * Copyright (c) 2020      Cisco Systems, Inc.  All rights reserved
- * Copyright (c) 2021-2022  Consulting.  All rights reserved.
- * Copyright (c) 2022-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -54,12 +53,10 @@ prte_state_base_module_t prte_state_prted_module = {
     .activate_job_state = prte_state_base_activate_job_state,
     .add_job_state = prte_state_base_add_job_state,
     .set_job_state_callback = prte_state_base_set_job_state_callback,
-    .set_job_state_priority = prte_state_base_set_job_state_priority,
     .remove_job_state = prte_state_base_remove_job_state,
     .activate_proc_state = prte_state_base_activate_proc_state,
     .add_proc_state = prte_state_base_add_proc_state,
     .set_proc_state_callback = prte_state_base_set_proc_state_callback,
-    .set_proc_state_priority = prte_state_base_set_proc_state_priority,
     .remove_proc_state = prte_state_base_remove_proc_state
 };
 
@@ -108,19 +105,18 @@ static int init(void)
 
     num_states = sizeof(job_states) / sizeof(prte_job_state_t);
     for (i = 0; i < num_states; i++) {
-        rc = prte_state.add_job_state(job_states[i], job_callbacks[i], PRTE_SYS_PRI);
+        rc = prte_state.add_job_state(job_states[i], job_callbacks[i]);
         if (PRTE_SUCCESS != rc) {
             PRTE_ERROR_LOG(rc);
         }
     }
     /* add a default error response */
-    rc = prte_state.add_job_state(PRTE_JOB_STATE_FORCED_EXIT, prte_quit, PRTE_ERROR_PRI);
+    rc = prte_state.add_job_state(PRTE_JOB_STATE_FORCED_EXIT, prte_quit);
     if (PRTE_SUCCESS != rc) {
         PRTE_ERROR_LOG(rc);
     }
     /* add a state for when we are ordered to terminate */
-    rc = prte_state.add_job_state(PRTE_JOB_STATE_DAEMONS_TERMINATED, prte_quit,
-                                  PRTE_SYS_PRI);
+    rc = prte_state.add_job_state(PRTE_JOB_STATE_DAEMONS_TERMINATED, prte_quit);
     if (PRTE_SUCCESS != rc) {
         PRTE_ERROR_LOG(rc);
     }
@@ -133,7 +129,7 @@ static int init(void)
      */
     num_states = sizeof(proc_states) / sizeof(prte_proc_state_t);
     for (i = 0; i < num_states; i++) {
-        rc = prte_state.add_proc_state(proc_states[i], proc_callbacks[i], PRTE_SYS_PRI);
+        rc = prte_state.add_proc_state(proc_states[i], proc_callbacks[i]);
         if (PRTE_SUCCESS != rc) {
             PRTE_ERROR_LOG(rc);
         }
@@ -479,11 +475,6 @@ static void track_procs(int fd, short argc, void *cbdata)
         PRTE_FLAG_SET(pdata, PRTE_PROC_FLAG_RECORDED);
         PRTE_FLAG_UNSET(pdata, PRTE_PROC_FLAG_ALIVE);
         pdata->state = state;
-        /* Clean up the session directory as if we were the process
-         * itself.  This covers the case where the process died abnormally
-         * and didn't cleanup its own session directory.
-         */
-        prte_session_dir_finalize(proc);
         /* if we are trying to terminate and our routes are
          * gone, then terminate ourselves IF no local procs
          * remain (might be some from another job)
