@@ -18,7 +18,7 @@
  *                         All rights reserved.
  * Copyright (c) 2014-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting.  All rights reserved.
  * Copyright (c) 2023      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
@@ -529,10 +529,6 @@ static void lost_connection_hdlr(size_t evhdlr_registration_id, pmix_status_t st
     PMIX_LIST_FOREACH(tl, &prte_pmix_server_globals.tools, prte_pmix_tool_t)
     {
         if (PMIX_CHECK_PROCID(&tl->name, source)) {
-            /* remove the session directory we created for it */
-            if (NULL != tl->nsdir) {
-                pmix_os_dirpath_destroy(tl->nsdir, true, NULL);
-            }
             /* take this tool off the list */
             pmix_list_remove_item(&prte_pmix_server_globals.tools, &tl->super);
             /* release it */
@@ -568,6 +564,7 @@ int pmix_server_init(void)
     pmix_data_array_t darray;
     pmix_info_t *info, myinf;
     size_t n, ninfo;
+    pmix_value_t *val;
     char *tmp;
     pmix_status_t prc;
     prte_pmix_lock_t lock;
@@ -636,7 +633,7 @@ int pmix_server_init(void)
 
     /* tell the server our temp directory */
     PMIX_INFO_LIST_ADD(prc, ilist, PMIX_SERVER_TMPDIR,
-                       prte_process_info.jobfam_session_dir,
+                       prte_process_info.top_session_dir,
                        PMIX_STRING);
     if (PMIX_SUCCESS != prc) {
         PMIX_INFO_LIST_RELEASE(ilist);
@@ -829,6 +826,24 @@ int pmix_server_init(void)
     PMIX_INFO_FREE(info, ninfo);
     rc = PRTE_SUCCESS;
 
+#ifdef PMIX_VERSION_NUMERIC
+    /* find out what version of PMIx is being used - note that
+     * it is NOT an error to not be able to retrieve this
+     * value as it just means the PMIx library pre-dates
+     * introduction of the ability to retrieve the version */
+    prc = PMIx_Get(NULL, PMIX_VERSION_NUMERIC, NULL, 0, &val);
+    if (PMIX_SUCCESS == prc) {
+        // check the version
+        if (val->data.uint32 < PRTE_PMIX_MINIMUM_VERSION) {
+            pmix_show_help("help-prted.txt", "min-pmix-violation", true,
+                           PRTE_PMIX_MINIMUM_VERSION, val->data.uint32);
+            PMIX_VALUE_RELEASE(val);
+            return PRTE_ERR_SILENT;
+        }
+        PMIX_VALUE_RELEASE(val);
+    }
+#endif
+
     /* register our support */
     for (n = 0; 0 != strlen(prte_attributes[n].function); n++) {
         prc = PMIx_Register_attributes(prte_attributes[n].function, prte_attributes[n].attrs);
@@ -1107,7 +1122,6 @@ static void modex_resp(pmix_status_t status, char *data, size_t sz, void *cbdata
         req->sz = sz;
     }
     prte_event_set(prte_event_base, &(req->ev), -1, PRTE_EV_WRITE, _mdxresp, req);
-    prte_event_set_priority(&(req->ev), PRTE_MSG_PRI);
     PMIX_POST_OBJECT(req);
     prte_event_active(&(req->ev), PRTE_EV_WRITE, 1);
 }
@@ -1206,10 +1220,11 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
     pmix_server_req_t *req;
     pmix_proc_t pproc;
     pmix_status_t prc;
-    pmix_info_t *info = NULL;
+    pmix_info_t *info = NULL, *iptr;
     size_t ninfo;
     char *key = NULL;
-    size_t sz;
+    size_t sz, n, refreshidx;
+    bool refresh_cache = false;
     pmix_value_t *pval = NULL;
     PRTE_HIDE_UNUSED_PARAMS(status, tg, cbdata);
 
@@ -1247,7 +1262,7 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
     if (NULL != info) {
         for (sz = 0; sz < ninfo; sz++) {
             if (PMIX_CHECK_KEY(&info[sz], PMIX_REQUIRED_KEY)) {
-                key = info[sz].value.data.string;
+                key = strdup(info[sz].value.data.string);
                 continue;
             }
             if (PMIX_CHECK_KEY(&info[sz], PMIX_TIMEOUT)) {
@@ -1261,6 +1276,34 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
                 }
                 continue;
             }
+            if (PMIX_CHECK_KEY(&info[sz], PMIX_GET_REFRESH_CACHE)) {
+                refresh_cache = PMIX_INFO_TRUE(&info[sz]);
+                refreshidx = sz;
+                continue;
+            }
+        }
+    }
+
+    if (refresh_cache) {
+        if (1 < ninfo) {
+            // need to remove the refresh cache key to avoid loops
+            PMIX_INFO_CREATE(iptr, ninfo - 1);
+            sz = 0;
+            for (n = 0; n < ninfo; n++) {
+                if (n == refreshidx) {
+                    continue;
+                }
+                PMIX_INFO_XFER(&iptr[sz], &info[n]);
+                ++sz;
+            }
+            PMIX_INFO_FREE(info, ninfo);
+            info = iptr;
+            ninfo = sz;
+        } else {
+            // refresh was the only key
+            PMIX_INFO_FREE(info, ninfo);
+            info = NULL;
+            ninfo = 0;
         }
     }
 
@@ -1281,7 +1324,8 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
         req->info = info;
         req->ninfo = ninfo;
         if (NULL != key) {
-            req->key = strdup(key);
+            req->key = key;
+            key = NULL;
         }
         /* store THEIR index to the request */
         req->remote_index = index;
@@ -1291,7 +1335,6 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
 
         /* setup the cycle timer so we periodically wake up and try again */
         prte_event_evtimer_set(prte_event_base, &req->cycle, dmdx_check, req);
-        prte_event_set_priority(&req->cycle, PRTE_MSG_PRI);
         req->cycle_active = true;
         PMIX_POST_OBJECT(req);
         tv.tv_sec = 2;
@@ -1300,7 +1343,6 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
         /* if they asked for a timeout, then set that too */
         if (0 < timeout) {
             prte_event_evtimer_set(prte_event_base, &req->ev, timeout_cbfunc, req);
-            prte_event_set_priority(&req->ev, PRTE_MSG_PRI);
             req->event_active = true;
             PMIX_POST_OBJECT(req);
             tv.tv_sec = timeout;
@@ -1323,7 +1365,8 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
     }
 
     if (NULL != key) {
-        pmix_output_verbose(2, prte_pmix_server_globals.output, "%s dmdx:recv checking for key %s",
+        pmix_output_verbose(2, prte_pmix_server_globals.output,
+                            "%s dmdx:recv checking for key %s",
                             PRTE_NAME_PRINT(PRTE_PROC_MY_NAME), key);
         /* see if we have it */
         if (PMIX_SUCCESS != PMIx_Get(&pproc, key, info, ninfo, &pval)) {
@@ -1337,7 +1380,8 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
             memcpy(&req->tproc, &pproc, sizeof(pmix_proc_t));
             req->info = info;
             req->ninfo = ninfo;
-            req->key = strdup(key);
+            req->key = key;
+            key = NULL;
             req->remote_index = index;
             /* store it in my remote reqs, assigning the index in that array
              * to the req->local_index as this is MY index to the request */
@@ -1345,7 +1389,6 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
 
             /* setup the cycle timer so we periodically wake up and try again */
             prte_event_evtimer_set(prte_event_base, &req->cycle, dmdx_check, req);
-            prte_event_set_priority(&req->cycle, PRTE_MSG_PRI);
             req->cycle_active = true;
             PMIX_POST_OBJECT(req);
             tv.tv_sec = 2;
@@ -1354,7 +1397,6 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
             /* if they asked for a timeout, then set that too */
             if (0 < timeout) {
                 prte_event_evtimer_set(prte_event_base, &req->ev, timeout_cbfunc, req);
-                prte_event_set_priority(&req->ev, PRTE_MSG_PRI);
                 req->event_active = true;
                 PMIX_POST_OBJECT(req);
                 tv.tv_sec = timeout;
@@ -1369,6 +1411,11 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
                             PRTE_NAME_PRINT(PRTE_PROC_MY_NAME), key);
     }
 
+    if (NULL != key) {
+        free(key);
+        key = NULL;
+    }
+
     /* track the request since the call down to the PMIx server
      * is asynchronous */
     req = PMIX_NEW(pmix_server_req_t);
@@ -1385,7 +1432,6 @@ static void pmix_server_dmdx_recv(int status, pmix_proc_t *sender,
     /* if they asked for a timeout, then set that too */
     if (0 < timeout) {
         prte_event_evtimer_set(prte_event_base, &req->ev, timeout_cbfunc, req);
-        prte_event_set_priority(&req->ev, PRTE_MSG_PRI);
         req->event_active = true;
         PMIX_POST_OBJECT(req);
         tv.tv_sec = timeout;
@@ -1974,16 +2020,6 @@ PMIX_CLASS_INSTANCE(pmix_server_pset_t,
                     pmix_list_item_t,
                     pscon, psdes);
 
-static void tlcon(prte_pmix_tool_t *p)
-{
-    p->nsdir = NULL;
-}
-static void tldes(prte_pmix_tool_t *p)
-{
-    if (NULL != p->nsdir) {
-        free(p->nsdir);
-    }
-}
 PMIX_CLASS_INSTANCE(prte_pmix_tool_t,
                     pmix_list_item_t,
-                    tlcon, tldes);
+                    NULL, NULL);
