@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,9 +11,13 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,8 +26,8 @@
  */
 
 #include "ompi_config.h"
-#include "opal/threads/mutex.h"
-#include "opal/threads/condition.h"
+#include "opal/mca/threads/mutex.h"
+#include "opal/mca/threads/condition.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "opal/mca/allocator/base/base.h"
 #include "opal/mca/allocator/allocator.h"
@@ -31,6 +36,7 @@
 #include "ompi/mca/pml/base/pml_base_sendreq.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "opal/mca/mpool/mpool.h"
+#include "ompi/runtime/mpiruntime.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -41,25 +47,24 @@ static opal_condition_t mca_pml_bsend_condition;  /* condition variable to block
 static mca_allocator_base_component_t* mca_pml_bsend_allocator_component;
 static mca_allocator_base_module_t* mca_pml_bsend_allocator;  /* sub-allocator to manage users buffer */
 static size_t           mca_pml_bsend_usersize;   /* user provided buffer size */
-unsigned char          *mca_pml_bsend_userbase = NULL; /* user provided buffer base */
-unsigned char          *mca_pml_bsend_base = NULL;     /* adjusted base of user buffer */
-unsigned char          *mca_pml_bsend_addr = NULL;     /* current offset into user buffer */
+unsigned char          *mca_pml_bsend_userbase=NULL;/* user provided buffer base */
+unsigned char          *mca_pml_bsend_base = NULL;/* adjusted base of user buffer */
+unsigned char          *mca_pml_bsend_addr = NULL;/* current offset into user buffer */
 static size_t           mca_pml_bsend_size;       /* adjusted size of user buffer */
 static size_t           mca_pml_bsend_count;      /* number of outstanding requests */
 static size_t           mca_pml_bsend_pagesz;     /* mmap page size */
 static int              mca_pml_bsend_pagebits;   /* number of bits in pagesz */
-static int32_t          mca_pml_bsend_init = 0;
+static opal_atomic_int32_t          mca_pml_bsend_init = 0;
 
 /* defined in pml_base_open.c */
 extern char *ompi_pml_base_bsend_allocator_name;
 
+static int mca_pml_base_bsend_fini (void);
+
 /*
  * Routine to return pages to sub-allocator as needed
  */
-static void* mca_pml_bsend_alloc_segment(
-    struct mca_mpool_base_module_t* module,
-    size_t* size_inout,
-    mca_mpool_base_registration_t** registration)
+static void* mca_pml_bsend_alloc_segment(void *ctx, size_t *size_inout)
 {
     void *addr;
     size_t size = *size_inout;
@@ -71,18 +76,17 @@ static void* mca_pml_bsend_alloc_segment(
     addr = mca_pml_bsend_addr;
     mca_pml_bsend_addr += size;
     *size_inout = size;
-    if (NULL != registration) *registration = NULL;
     return addr;
 }
 
 /*
  * One time initialization at startup
  */
-int mca_pml_base_bsend_init(bool thread_safe)
+int mca_pml_base_bsend_init (void)
 {
     size_t tmp;
 
-    if(OPAL_THREAD_ADD32(&mca_pml_bsend_init, 1) > 1)
+    if(OPAL_THREAD_ADD_FETCH32(&mca_pml_bsend_init, 1) > 1)
         return OMPI_SUCCESS;
 
     /* initialize static objects */
@@ -101,6 +105,9 @@ int mca_pml_base_bsend_init(bool thread_safe)
         tmp >>= 1;
         mca_pml_bsend_pagebits++;
     }
+
+    ompi_mpi_instance_append_finalize (mca_pml_base_bsend_fini);
+
     return OMPI_SUCCESS;
 }
 
@@ -108,9 +115,9 @@ int mca_pml_base_bsend_init(bool thread_safe)
 /*
  * One-time cleanup at shutdown - release any resources.
  */
-int mca_pml_base_bsend_fini(void)
+static int mca_pml_base_bsend_fini (void)
 {
-    if(OPAL_THREAD_ADD32(&mca_pml_bsend_init,-1) > 0)
+    if(OPAL_THREAD_ADD_FETCH32(&mca_pml_bsend_init,-1) > 0)
         return OMPI_SUCCESS;
 
     if(NULL != mca_pml_bsend_allocator)
@@ -233,7 +240,7 @@ int mca_pml_base_bsend_request_start(ompi_request_t* request)
 
         /* allocate a buffer to hold packed message */
         sendreq->req_addr = mca_pml_bsend_allocator->alc_alloc(
-            mca_pml_bsend_allocator, sendreq->req_bytes_packed, 0, NULL);
+            mca_pml_bsend_allocator, sendreq->req_bytes_packed, 0);
         if(NULL == sendreq->req_addr) {
             /* release resources when request is freed */
             sendreq->req_base.req_pml_complete = true;
@@ -243,7 +250,7 @@ int mca_pml_base_bsend_request_start(ompi_request_t* request)
 
         OPAL_THREAD_UNLOCK(&mca_pml_bsend_mutex);
 
-        /* The convertor is already initialized in the begining so we just have to
+        /* The convertor is already initialized in the beginning so we just have to
          * pack the data in the newly allocated buffer.
          */
         iov.iov_base = (IOVBASE_TYPE*)sendreq->req_addr;
@@ -288,7 +295,7 @@ int mca_pml_base_bsend_request_alloc(ompi_request_t* request)
 
     /* allocate a buffer to hold packed message */
     sendreq->req_addr = mca_pml_bsend_allocator->alc_alloc(
-        mca_pml_bsend_allocator, sendreq->req_bytes_packed, 0, NULL);
+        mca_pml_bsend_allocator, sendreq->req_bytes_packed, 0);
     if(NULL == sendreq->req_addr) {
         /* release resources when request is freed */
         sendreq->req_base.req_pml_complete = true;
@@ -322,7 +329,7 @@ void*  mca_pml_base_bsend_request_alloc_buf( size_t length )
 
     /* allocate a buffer to hold packed message */
     buf = mca_pml_bsend_allocator->alc_alloc(
-        mca_pml_bsend_allocator, length, 0, NULL);
+        mca_pml_bsend_allocator, length, 0);
     if(NULL == buf) {
         /* release resources when request is freed */
         OPAL_THREAD_UNLOCK(&mca_pml_bsend_mutex);
