@@ -14,8 +14,12 @@
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved
  * Copyright (c) 2013-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2018      Sandia National Laboratories
+ *                         All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2020      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,7 +28,7 @@
  */
 
 #include "ompi_config.h"
-#include "opal/mca/event/event.h"
+#include "opal/util/event.h"
 #include "mpi.h"
 #include "ompi/runtime/params.h"
 #include "ompi/mca/pml/pml.h"
@@ -35,6 +39,7 @@
 #include "pml_ob1_recvreq.h"
 #include "pml_ob1_rdmafrag.h"
 #include "pml_ob1_recvfrag.h"
+#include "pml_ob1_accelerator.h"
 #include "ompi/mca/bml/base/base.h"
 #include "pml_ob1_component.h"
 #include "opal/mca/allocator/base/base.h"
@@ -58,12 +63,12 @@ int mca_pml_ob1_output = 0;
 static int mca_pml_ob1_verbose = 0;
 bool mca_pml_ob1_matching_protection = false;
 
-mca_pml_base_component_2_0_0_t mca_pml_ob1_component = {
+mca_pml_base_component_2_1_0_t mca_pml_ob1_component = {
     /* First, the mca_base_component_t struct containing meta
        information about the component itself */
 
     .pmlm_version = {
-        MCA_PML_BASE_VERSION_2_0_0,
+        MCA_PML_BASE_VERSION_2_1_0,
 
         .mca_component_name = "ob1",
         .mca_component_major_version = OMPI_MAJOR_VERSION,
@@ -149,7 +154,12 @@ static int mca_pml_ob1_get_unex_msgq_size (const struct mca_base_pvar_t *pvar, v
     for (i = 0 ; i < comm_size ; ++i) {
         pml_proc = pml_comm->procs[i];
         if (pml_proc) {
+#if MCA_PML_OB1_CUSTOM_MATCH
+            values[i] = custom_match_umq_size(pml_comm->umq); // TODO: given the structure of custom match this does not make sense,
+                                                     //       as we only have one set of queues.
+#else
             values[i] = opal_list_get_size (&pml_proc->unexpected_frags);
+#endif
         } else {
             values[i] = 0;
         }
@@ -171,7 +181,12 @@ static int mca_pml_ob1_get_posted_recvq_size (const struct mca_base_pvar_t *pvar
         pml_proc = pml_comm->procs[i];
 
         if (pml_proc) {
+#if MCA_PML_OB1_CUSTOM_MATCH
+            values[i] = custom_match_prq_size(pml_comm->prq); // TODO: given the structure of custom match this does not make sense,
+                                                     //       as we only have one set of queues.
+#else
             values[i] = opal_list_get_size (&pml_proc->specific_receives);
+#endif
         } else {
             values[i] = 0;
         }
@@ -300,6 +315,9 @@ mca_pml_ob1_component_init( int* priority,
 
     }
 
+    /** this pml supports the extended CID space */
+    mca_pml_ob1.super.pml_flags |= MCA_PML_BASE_FLAG_SUPPORTS_EXT_CID;
+
     return &mca_pml_ob1.super;
 }
 
@@ -345,6 +363,10 @@ int mca_pml_ob1_component_fini(void)
     OBJ_DESTRUCT(&mca_pml_ob1.lock);
     OBJ_DESTRUCT(&mca_pml_ob1.send_ranges);
 
+    if (mca_pml_ob1.accelerator_enabled) {
+        mca_pml_ob1_accelerator_fini();
+    }
+
     if( NULL != mca_pml_ob1.allocator ) {
         (void)mca_pml_ob1.allocator->alc_finalize(mca_pml_ob1.allocator);
         mca_pml_ob1.allocator = NULL;
