@@ -3,8 +3,12 @@
  * Copyright (c) 2016-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2018      Research Organization for Information Science
+ * Copyright (c) 2018-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2018-2022 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2019      Intel, Inc.  All rights reserved.
+ * Copyright (c) 2022      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -15,68 +19,74 @@
 #include "pml_ucx.h"
 
 #include "opal/runtime/opal.h"
-#include "opal/mca/pmix/pmix.h"
+#include "opal/mca/pmix/pmix-internal.h"
 #include "ompi/attribute/attribute.h"
 #include "ompi/message/message.h"
+#include "ompi/runtime/ompi_spc.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "opal/mca/common/ucx/common_ucx.h"
-#if OPAL_CUDA_SUPPORT
-#include "opal/mca/common/cuda/common_cuda.h"
-#endif /* OPAL_CUDA_SUPPORT */
 #include "pml_ucx_request.h"
 
 #include <inttypes.h>
 
+/*
+ * Maximum number of transport entries that we can expect from UCX
+ */
+#define PML_UCX_MAX_TRANSPORT_ENTRIES 100
 
 #define PML_UCX_TRACE_SEND(_msg, _buf, _count, _datatype, _dst, _tag, _mode, _comm, ...) \
     PML_UCX_VERBOSE(8, _msg " buf %p count %zu type '%s' dst %d tag %d mode %s comm %d '%s'", \
                     __VA_ARGS__, \
                     (_buf), (_count), (_datatype)->name, (_dst), (_tag), \
-                    mca_pml_ucx_send_mode_name(_mode), (_comm)->c_contextid, \
+                    mca_pml_ucx_send_mode_name(_mode), (_comm)->c_index, \
                     (_comm)->c_name);
 
 #define PML_UCX_TRACE_RECV(_msg, _buf, _count, _datatype, _src, _tag, _comm, ...) \
     PML_UCX_VERBOSE(8, _msg " buf %p count %zu type '%s' src %d tag %d comm %d '%s'", \
                     __VA_ARGS__, \
                     (_buf), (_count), (_datatype)->name, (_src), (_tag), \
-                    (_comm)->c_contextid, (_comm)->c_name);
+                    (_comm)->c_index, (_comm)->c_name);
 
 #define PML_UCX_TRACE_PROBE(_msg, _src, _tag, _comm) \
     PML_UCX_VERBOSE(8, _msg " src %d tag %d comm %d '%s'", \
-                    _src, (_tag), (_comm)->c_contextid, (_comm)->c_name);
+                    _src, (_tag), (_comm)->c_index, (_comm)->c_name);
 
 #define PML_UCX_TRACE_MRECV(_msg, _buf, _count, _datatype, _message) \
     PML_UCX_VERBOSE(8, _msg " buf %p count %zu type '%s' msg *%p=%p (%p)", \
                     (_buf), (_count), (_datatype)->name, (void*)(_message), \
                     (void*)*(_message), (*(_message))->req_ptr);
 
+mca_pml_transports_t *mca_pml_ucx_get_transports(ompi_communicator_t *comm,
+                                                 int rank);
+
 #define MODEX_KEY "pml-ucx"
 
 mca_pml_ucx_module_t ompi_pml_ucx = {
     .super = {
-        .pml_add_procs     = mca_pml_ucx_add_procs,
-        .pml_del_procs     = mca_pml_ucx_del_procs,
-        .pml_enable        = mca_pml_ucx_enable,
-        .pml_progress      = NULL,
-        .pml_add_comm      = mca_pml_ucx_add_comm,
-        .pml_del_comm      = mca_pml_ucx_del_comm,
-        .pml_irecv_init    = mca_pml_ucx_irecv_init,
-        .pml_irecv         = mca_pml_ucx_irecv,
-        .pml_recv          = mca_pml_ucx_recv,
-        .pml_isend_init    = mca_pml_ucx_isend_init,
-        .pml_isend         = mca_pml_ucx_isend,
-        .pml_send          = mca_pml_ucx_send,
-        .pml_iprobe        = mca_pml_ucx_iprobe,
-        .pml_probe         = mca_pml_ucx_probe,
-        .pml_start         = mca_pml_ucx_start,
-        .pml_improbe       = mca_pml_ucx_improbe,
-        .pml_mprobe        = mca_pml_ucx_mprobe,
-        .pml_imrecv        = mca_pml_ucx_imrecv,
-        .pml_mrecv         = mca_pml_ucx_mrecv,
-        .pml_dump          = mca_pml_ucx_dump,
-        .pml_ft_event      = NULL,
-        .pml_max_contextid = (1ul << (PML_UCX_CONTEXT_BITS)) - 1,
-        .pml_max_tag       = (1ul << (PML_UCX_TAG_BITS - 1)) - 1
+        .pml_add_procs      = mca_pml_ucx_add_procs,
+        .pml_del_procs      = mca_pml_ucx_del_procs,
+        .pml_enable         = mca_pml_ucx_enable,
+        .pml_progress       = NULL,
+        .pml_add_comm       = mca_pml_ucx_add_comm,
+        .pml_del_comm       = mca_pml_ucx_del_comm,
+        .pml_irecv_init     = mca_pml_ucx_irecv_init,
+        .pml_irecv          = mca_pml_ucx_irecv,
+        .pml_recv           = mca_pml_ucx_recv,
+        .pml_isend_init     = mca_pml_ucx_isend_init,
+        .pml_isend          = mca_pml_ucx_isend,
+        .pml_send           = mca_pml_ucx_send,
+        .pml_iprobe         = mca_pml_ucx_iprobe,
+        .pml_probe          = mca_pml_ucx_probe,
+        .pml_start          = mca_pml_ucx_start,
+        .pml_improbe        = mca_pml_ucx_improbe,
+        .pml_mprobe         = mca_pml_ucx_mprobe,
+        .pml_imrecv         = mca_pml_ucx_imrecv,
+        .pml_mrecv          = mca_pml_ucx_mrecv,
+        .pml_dump           = mca_pml_ucx_dump,
+        .pml_max_contextid  = (1ul << (PML_UCX_CONTEXT_BITS)) - 1,
+        .pml_max_tag        = (1ul << (PML_UCX_TAG_BITS - 1)) - 1,
+        .pml_flags          = 0, /* flags */
+        .pml_get_transports = mca_pml_ucx_get_transports
     },
     .ucp_context           = NULL,
     .ucp_worker            = NULL
@@ -112,7 +122,7 @@ static int mca_pml_ucx_send_worker_address_type(int addr_flags, int modex_scope)
     }
 
     PML_UCX_VERBOSE(2, "Pack %s worker address, size %ld",
-                    (modex_scope == OPAL_PMIX_LOCAL) ? "local" : "remote",
+                    (modex_scope == PMIX_LOCAL) ? "local" : "remote",
                     attrs.address_length);
 
     return OMPI_SUCCESS;
@@ -136,7 +146,7 @@ static int mca_pml_ucx_send_worker_address(void)
 
     PML_UCX_VERBOSE(2, "Pack worker address, size %ld", addrlen);
 
-    OPAL_MODEX_SEND(rc, OPAL_PMIX_GLOBAL,
+    OPAL_MODEX_SEND(rc, PMIX_GLOBAL,
                     &mca_pml_ucx_component.pmlm_version, (void*)address, addrlen);
 
     ucp_worker_release_address(ompi_pml_ucx.ucp_worker, address);
@@ -147,12 +157,12 @@ static int mca_pml_ucx_send_worker_address(void)
 #else
     /* Pack just network device addresses for remote node peers */
     status = mca_pml_ucx_send_worker_address_type(UCP_WORKER_ADDRESS_FLAG_NET_ONLY,
-                                                  OPAL_PMIX_REMOTE);
+                                                  PMIX_REMOTE);
     if (UCS_OK != status) {
         goto err;
     }
 
-    status = mca_pml_ucx_send_worker_address_type(0, OPAL_PMIX_LOCAL);
+    status = mca_pml_ucx_send_worker_address_type(0, PMIX_LOCAL);
     if (UCS_OK != status) {
         goto err;
     }
@@ -204,6 +214,12 @@ int mca_pml_ucx_open(void)
         return OMPI_ERROR;
     }
 
+    if ((major_version == 1) && (minor_version < 9)) {
+        /* show warning due to issue #8549 */
+        PML_UCX_WARN("UCX version %u.%u.%u is too old, please install 1.9.x or "
+                     "newer", major_version, minor_version, release_number);
+    }
+
     /* Read options */
     status = ucp_config_read("MPI", NULL, &config);
     if (UCS_OK != status) {
@@ -252,14 +268,7 @@ int mca_pml_ucx_open(void)
     }
 
     ompi_pml_ucx.request_size     = attr.request_size;
-    ompi_pml_ucx.cuda_initialized = false;
 
-#if HAVE_UCP_ATTR_MEMORY_TYPES && OPAL_CUDA_SUPPORT
-    if (attr.memory_types & UCS_BIT(UCS_MEMORY_TYPE_CUDA)) {
-        mca_common_cuda_stage_one_init();
-        ompi_pml_ucx.cuda_initialized = true;
-    }
-#endif
     return OMPI_SUCCESS;
 }
 
@@ -267,11 +276,6 @@ int mca_pml_ucx_close(void)
 {
     PML_UCX_VERBOSE(1, "mca_pml_ucx_close");
 
-#if OPAL_CUDA_SUPPORT
-    if (ompi_pml_ucx.cuda_initialized) {
-        mca_common_cuda_fini();
-    }
-#endif
     if (ompi_pml_ucx.ucp_context != NULL) {
         ucp_cleanup(ompi_pml_ucx.ucp_context);
         ompi_pml_ucx.ucp_context = NULL;
@@ -288,12 +292,12 @@ int mca_pml_ucx_init(int enable_mpi_threads)
 
     PML_UCX_VERBOSE(1, "mca_pml_ucx_init");
 
+    /* TODO check MPI thread mode */
     params.field_mask  = UCP_WORKER_PARAM_FIELD_THREAD_MODE;
     if (enable_mpi_threads) {
         params.thread_mode = UCS_THREAD_MODE_MULTI;
     } else {
-        params.thread_mode =
-            opal_common_ucx_thread_mode(ompi_mpi_thread_provided);
+        params.thread_mode = UCS_THREAD_MODE_SINGLE;
     }
 
 #if HAVE_DECL_UCP_WORKER_FLAG_IGNORE_REQUEST_LEAK
@@ -322,8 +326,8 @@ int mca_pml_ucx_init(int enable_mpi_threads)
     if (enable_mpi_threads && (attr.thread_mode != UCS_THREAD_MODE_MULTI)) {
         /* UCX does not support multithreading, disqualify current PML for now */
         /* TODO: we should let OMPI to fallback to THREAD_SINGLE mode */
-        PML_UCX_VERBOSE(1, "UCP worker does not support MPI_THREAD_MULTIPLE. "
-                           "PML UCX could not be selected");
+        PML_UCX_WARN("UCP worker does not support MPI_THREAD_MULTIPLE. "
+                     "PML UCX could not be selected");
         rc = OMPI_ERR_NOT_SUPPORTED;
         goto err_destroy_worker;
     }
@@ -345,19 +349,22 @@ int mca_pml_ucx_init(int enable_mpi_threads)
     /* Create a completed request to be returned from isend */
     OBJ_CONSTRUCT(&ompi_pml_ucx.completed_send_req, ompi_request_t);
     mca_pml_ucx_completed_request_init(&ompi_pml_ucx.completed_send_req);
+#if MPI_VERSION >= 4
+    ompi_pml_ucx.completed_send_req.req_cancel = mca_pml_cancel_send_callback;
+#endif
 
     opal_progress_register(mca_pml_ucx_progress);
 
     PML_UCX_VERBOSE(2, "created ucp context %p, worker %p",
                     (void *)ompi_pml_ucx.ucp_context,
                     (void *)ompi_pml_ucx.ucp_worker);
-    return rc;
+    return OMPI_SUCCESS;
 
 err_destroy_worker:
     ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
-    ompi_pml_ucx.ucp_worker = NULL;
 err:
-    return OMPI_ERROR;
+    ompi_pml_ucx.ucp_worker = NULL;
+    return rc;
 }
 
 int mca_pml_ucx_cleanup(void)
@@ -386,7 +393,7 @@ int mca_pml_ucx_cleanup(void)
     OBJ_DESTRUCT(&ompi_pml_ucx.convs);
     OBJ_DESTRUCT(&ompi_pml_ucx.persistent_reqs);
 
-    if (ompi_pml_ucx.ucp_worker) {
+    if (ompi_pml_ucx.ucp_worker != NULL) {
         ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
         ompi_pml_ucx.ucp_worker = NULL;
     }
@@ -431,12 +438,6 @@ static ucp_ep_h mca_pml_ucx_add_proc_common(ompi_proc_t *proc)
     return ep;
 }
 
-static ucp_ep_h mca_pml_ucx_add_proc(ompi_communicator_t *comm, int dst)
-{
-    ompi_proc_t *proc_peer = ompi_comm_peer_lookup(comm, dst);
-    return mca_pml_ucx_add_proc_common(proc_peer);
-}
-
 int mca_pml_ucx_add_procs(struct ompi_proc_t **procs, size_t nprocs)
 {
     ompi_proc_t *proc;
@@ -455,29 +456,77 @@ int mca_pml_ucx_add_procs(struct ompi_proc_t **procs, size_t nprocs)
     return OMPI_SUCCESS;
 }
 
+mca_pml_transports_t *mca_pml_ucx_get_transports(struct ompi_communicator_t *comm,
+                                                 int rank)
+{
+#if HAVE_DECL_UCP_EP_ATTR_FIELD_TRANSPORTS
+    mca_pml_transports_t *transports;
+    unsigned i;
+    ucs_status_t status;
+    ucp_ep_attr_t ep_attrs;
+    ompi_proc_t *proc = ompi_comm_peer_lookup(comm, rank);
+
+    if (NULL == proc) {
+        return NULL;
+    }
+    if (NULL == proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML]) {
+        return NULL;
+    }
+
+    ep_attrs.field_mask = UCP_EP_ATTR_FIELD_TRANSPORTS;
+    ep_attrs.transports.num_entries = PML_UCX_MAX_TRANSPORT_ENTRIES;
+    ep_attrs.transports.entry_size = sizeof(ucp_transport_entry_t);
+    ep_attrs.transports.entries = (ucp_transport_entry_t *) malloc(
+             sizeof(ucp_transport_entry_t) * ep_attrs.transports.num_entries);
+    if (NULL == ep_attrs.transports.entries) {
+        return NULL;
+    }
+
+    status = ucp_ep_query(proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML],
+                          &ep_attrs);
+    if (UCS_OK != status) {
+        return NULL;
+    }
+
+    transports = (mca_pml_transports_t *) malloc(sizeof(mca_pml_transports_t));
+    if (NULL == transports) {
+        return NULL;
+    }
+
+    transports->entries = (mca_pml_transport_entry_t *) malloc(
+        ep_attrs.transports.num_entries * sizeof(mca_pml_transport_entry_t));
+    if (NULL == transports->entries) {
+        free(transports);
+        return NULL;
+    }
+
+    for (i = 0; i < ep_attrs.transports.num_entries; i++) {
+        transports->entries[i].transport_name =
+                ep_attrs.transports.entries[i].transport_name;
+        transports->entries[i].device_name = 
+                ep_attrs.transports.entries[i].device_name;
+    }
+    transports->count = ep_attrs.transports.num_entries;
+
+    free(ep_attrs.transports.entries);
+    return transports;
+#else
+    return NULL;
+#endif
+}
+
 __opal_attribute_always_inline__
 static inline ucp_ep_h mca_pml_ucx_get_ep(ompi_communicator_t *comm, int rank)
 {
+    ompi_proc_t *proc_peer = ompi_comm_peer_lookup(comm, rank);
     ucp_ep_h ep;
 
-    ep = ompi_comm_peer_lookup(comm, rank)->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML];
-    if (OPAL_LIKELY(ep != NULL)) {
+    ep = proc_peer->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML];
+    if (OPAL_LIKELY(NULL != ep)) {
         return ep;
     }
 
-    ep = mca_pml_ucx_add_proc(comm, rank);
-    if (OPAL_LIKELY(ep != NULL)) {
-        return ep;
-    }
-
-    if (rank >= ompi_comm_size(comm)) {
-        PML_UCX_ERROR("Rank number (%d) is larger than communicator size (%d)",
-                      rank, ompi_comm_size(comm));
-    } else {
-        PML_UCX_ERROR("Failed to resolve UCX endpoint for rank %d", rank);
-    }
-
-    return NULL;
+    return mca_pml_ucx_add_proc_common(proc_peer);
 }
 
 int mca_pml_ucx_del_procs(struct ompi_proc_t **procs, size_t nprocs)
@@ -515,8 +564,7 @@ int mca_pml_ucx_enable(bool enable)
     int ret;
 
     /* Create a key for adding custom attributes to datatypes */
-    copy_fn.attr_datatype_copy_fn  =
-                    (MPI_Type_internal_copy_attr_function*)MPI_TYPE_NULL_COPY_FN;
+    copy_fn.attr_datatype_copy_fn  = MPI_TYPE_NULL_COPY_FN;
     del_fn.attr_datatype_delete_fn = mca_pml_ucx_datatype_attr_del_fn;
     ret = ompi_attr_create_keyval(TYPE_ATTR, copy_fn, del_fn,
                                   &ompi_pml_ucx.datatype_attr_keyval, NULL, 0,
@@ -635,6 +683,7 @@ int mca_pml_ucx_recv(void *buf, size_t count, ompi_datatype_t *datatype, int src
     ucp_tag_t ucp_tag, ucp_tag_mask;
     ucp_tag_recv_info_t info;
     ucs_status_t status;
+    int result;
 
     PML_UCX_TRACE_RECV("%s", buf, count, datatype, src, tag, comm, "recv");
 
@@ -651,7 +700,15 @@ int mca_pml_ucx_recv(void *buf, size_t count, ompi_datatype_t *datatype, int src
     MCA_COMMON_UCX_PROGRESS_LOOP(ompi_pml_ucx.ucp_worker) {
         status = ucp_request_test(req, &info);
         if (status != UCS_INPROGRESS) {
-            return mca_pml_ucx_set_recv_status_public(mpi_status, status, &info);
+            result = mca_pml_ucx_set_recv_status_public(mpi_status, status, &info);
+
+#if SPC_ENABLE == 1
+            size_t dt_size;
+            ompi_datatype_type_size(datatype, &dt_size);
+            SPC_USER_OR_MPI(tag, dt_size*count,
+                            OMPI_SPC_BYTES_RECEIVED_USER, OMPI_SPC_BYTES_RECEIVED_MPI);
+#endif
+            return result;
         }
     }
 }
@@ -852,6 +909,13 @@ int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
                                                    mca_pml_ucx_send_completion);
 #endif
 
+#if SPC_ENABLE == 1
+    size_t dt_size;
+    ompi_datatype_type_size(datatype, &dt_size);
+    SPC_USER_OR_MPI(tag, dt_size*count,
+                    OMPI_SPC_BYTES_SENT_USER, OMPI_SPC_BYTES_SENT_MPI);
+#endif
+
     if (req == NULL) {
         PML_UCX_VERBOSE(8, "returning completed request");
         *request = &ompi_pml_ucx.completed_send_req;
@@ -859,6 +923,11 @@ int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
     } else if (!UCS_PTR_IS_ERR(req)) {
         PML_UCX_VERBOSE(8, "got request %p", (void*)req);
         req->req_mpi_object.comm = comm;
+#if MPI_VERSION >= 4
+        if (OPAL_LIKELY(mca_pml_ucx_request_cancel == req->req_cancel)) {
+            req->req_cancel      = mca_pml_ucx_request_cancel_send;
+        }
+#endif
         *request                 = req;
         return OMPI_SUCCESS;
     } else {
@@ -943,6 +1012,13 @@ int mca_pml_ucx_send(const void *buf, size_t count, ompi_datatype_t *datatype, i
         return OMPI_ERROR;
     }
 
+#if SPC_ENABLE == 1
+    size_t dt_size;
+    ompi_datatype_type_size(datatype, &dt_size);
+    SPC_USER_OR_MPI(tag, dt_size*count,
+                    OMPI_SPC_BYTES_SENT_USER, OMPI_SPC_BYTES_SENT_MPI);
+#endif
+
 #if HAVE_DECL_UCP_TAG_SEND_NBR
     if (OPAL_LIKELY((MCA_PML_BASE_SEND_BUFFERED != mode) &&
                     (MCA_PML_BASE_SEND_SYNCHRONOUS != mode))) {
@@ -975,7 +1051,7 @@ int mca_pml_ucx_iprobe(int src, int tag, struct ompi_communicator_t* comm,
         mca_pml_ucx_set_recv_status_public(mpi_status, UCS_OK, &info);
     } else  {
         (++progress_count % opal_common_ucx.progress_iterations) ?
-            (void)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
+            (int)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
         *matched = 0;
     }
     return OMPI_SUCCESS;
@@ -1024,7 +1100,7 @@ int mca_pml_ucx_improbe(int src, int tag, struct ompi_communicator_t* comm,
         mca_pml_ucx_set_recv_status_public(mpi_status, UCS_OK, &info);
     } else  {
         (++progress_count % opal_common_ucx.progress_iterations) ?
-            (void)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
+            (int)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
         *matched = 0;
     }
     return OMPI_SUCCESS;
