@@ -5,6 +5,7 @@
  *                         All rights reserved.
  * Copyright (c) 2014-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,7 +28,7 @@
 /**
  * All-reduce - subgroup in communicator
  */
-OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
+OMPI_DECLSPEC int ompi_comm_allgather_pml(void *src_buf, void *dest_buf, int count,
         ompi_datatype_t *dtype, int my_rank_in_group,
         int n_peers, int *ranks_in_comm,ompi_communicator_t *comm)
 {
@@ -48,7 +49,7 @@ OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
     ompi_request_t *requests[4];
 
     /* get size of data needed - same layout as user data, so that
-     *   we can apply the reudction routines directly on these buffers
+     *   we can apply the reduction routines directly on these buffers
      */
     rc = ompi_datatype_type_size(dtype, &dt_size);
     if( OMPI_SUCCESS != rc ) {
@@ -76,7 +77,7 @@ OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
 
     /* get my reduction communication pattern */
     memset(&my_exchange_node, 0, sizeof(netpatterns_pair_exchange_node_t));
-    rc = netpatterns_setup_recursive_doubling_tree_node(n_peers,
+    rc = ompi_netpatterns_setup_recursive_doubling_tree_node(n_peers,
             my_rank_in_group, &my_exchange_node);
     if(OMPI_SUCCESS != rc){
         return rc;
@@ -164,7 +165,7 @@ OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
                  */
                 extra_end=n_extra_nodes-1;
             }
-            /* get the number of ranks whos data is to be grabbed */
+            /* get the number of ranks whose data is to be grabbed */
             n_extra=extra_end-extra_start+1;
 
             recv_iov[1].iov_base=(char *)dest_buf+
@@ -209,7 +210,7 @@ OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
                  */
                 extra_end=n_extra_nodes-1;
             }
-            /* get the number of ranks whos data is to be grabbed */
+            /* get the number of ranks whose data is to be grabbed */
             n_extra=extra_end-extra_start+1;
 
             send_iov[1].iov_base=(char *)dest_buf+
@@ -283,7 +284,7 @@ OMPI_DECLSPEC int comm_allgather_pml(void *src_buf, void *dest_buf, int count,
         }
     }
 
-    netpatterns_cleanup_recursive_doubling_tree_node(&my_exchange_node);
+    ompi_netpatterns_cleanup_recursive_doubling_tree_node(&my_exchange_node);
 
     /* return */
     return OMPI_SUCCESS;
