@@ -1,7 +1,14 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2009-2012 Mellanox Technologies.  All rights reserved.
  * Copyright (c) 2009-2012 Oak Ridge National Laboratory.  All rights reserved.
-* $COPYRIGHT$
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2019      Intel, Inc.  All rights reserved.
+ * $COPYRIGHT$
  *
  * Additional copyrights may follow
  *
@@ -21,11 +28,14 @@
 #include <assert.h>
 
 #include "ompi/constants.h"
+
+#include "ompi/runtime/ompi_rte.h"
+
 #include "netpatterns.h"
 
 /* setup recursive doubleing tree node */
 
-OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
+OMPI_DECLSPEC int ompi_netpatterns_setup_recursive_knomial_allgather_tree_node(
         int num_nodes, int node_rank, int tree_order, int *hier_ranks,
         netpatterns_k_exchange_node_t *exchange_node)
 {
@@ -37,14 +47,14 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
     int k_temp2;
     int myid, reindex_myid = 0;
     int base, peer_base,base_temp;
-    int peer; 
+    int peer;
     int *prev_data = NULL;
     int *current_data = NULL;
     int *group_info = NULL;
 
 
     NETPATTERNS_VERBOSE(
-            ("Enter netpatterns_setup_recursive_knomial_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)",
+            ("Enter ompi_netpatterns_setup_recursive_knomial_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)",
                 num_nodes, node_rank, tree_order));
 
     assert(num_nodes > 1);
@@ -56,8 +66,8 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
     /* k-nomial radix */
     exchange_node->tree_order = tree_order;
 
-    /* Calculate the number of levels in the tree for 
-     * the largest power of tree_order less than or 
+    /* Calculate the number of levels in the tree for
+     * the largest power of tree_order less than or
      * equal to the group size
      */
     n_levels = 0;
@@ -66,8 +76,8 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
         cnt *= tree_order;
         n_levels++;
     }
-    /* this is the actual number of recusive k-ing steps 
-     * we will perform, the last step may not be a full 
+    /* this is the actual number of recursive k-ing steps
+     * we will perform, the last step may not be a full
      * step depending on the outcome of the next conditional
      */
     pow_k = n_levels;
@@ -83,7 +93,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
     exchange_node->log_tree_order = pow_k;
     exchange_node->n_largest_pow_tree_order = cnt;
 
-    
+
     /* find the number of complete groups of size tree_order, tree_order^2, tree_order^3,...,tree_order^pow_k */
     /* I don't think we need to cache this info this group_info array */
     group_info = (int *) calloc(pow_k , sizeof(int));
@@ -102,11 +112,11 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
     }
     knt--;
     /*fprintf(stderr,"Maximal power of k is %d and the number of incomplete groups is %d \n", knt+1 ,tree_order - group_info[knt] );*/
-    
+
     /* k_temp is a synonym for cnt which is the largest full power of k group */
-    /* now, start the calculation to find the first stray rank aka "extra" rank */ 
+    /* now, start the calculation to find the first stray rank aka "extra" rank */
     stray = 0;
-    /*fprintf(stderr,"Maximal power of k %d, first stragler rank is %d and the number of straglers is %d\n",cnt, 
+    /*fprintf(stderr,"Maximal power of k %d, first stragler rank is %d and the number of straglers is %d\n",cnt,
                                                                            cnt*group_info[knt],
                                                                            num_nodes - cnt*group_info[knt]);*/
 
@@ -204,9 +214,9 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
         }
 
     }
-    /* intialize the payload array 
-       This is the money struct, just need to initialize this with 
-       the subgroup information */ 
+    /* initialize the payload array
+       This is the money struct, just need to initialize this with
+       the subgroup information */
     /*
     for(i = 0; i < num_nodes; i++){
         prev_data[i] = 1;
@@ -219,8 +229,8 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
         current_data[i] = hier_ranks[i];
     }
 
-    /* everyone will need to do this loop over all ranks 
-     * Phase I calculate the contribution from the extra ranks 
+    /* everyone will need to do this loop over all ranks
+     * Phase I calculate the contribution from the extra ranks
      */
     for( myid = 0; myid < num_nodes; myid++) {
         /* get my new rank */
@@ -260,7 +270,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
                     break;
                 }
             }
-            if( reindex_myid < stray ) { 
+            if( reindex_myid < stray ) {
                 /* now start the actual algorithm */
                 FIND_BASE(base,reindex_myid,i+1,tree_order);
                 for( j = 0; j < ( tree_order - 1 ); j ++ ) {
@@ -272,19 +282,19 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
                             exchange_node->payload_info[i][j].r_len = prev_data[exchange_node->reindex_map[peer]];
                             /*fprintf(stderr,"exchange_node->payload_info[%d][%d].r_len %d\n",i,j,prev_data[exchange_node->reindex_map[peer]]);*/
                             if( i > 0 ) {
-                                
+
                                 /* find my len and offset */
                                 FIND_BASE(peer_base,peer,i,tree_order);
                                 /* I do not want to mess with this, but it seems that I have no choice */
                                ex_node = exchange_node->reindex_map[peer_base];
                                /* now, find out how far down the line this guy really is */
                                knt2 =0;
-                               for(kk = 0; kk < ex_node; kk++){ 
+                               for(kk = 0; kk < ex_node; kk++){
                                    knt2 += hier_ranks[kk];
                                }
-                                exchange_node->payload_info[i][j].r_offset = knt2; 
+                                exchange_node->payload_info[i][j].r_offset = knt2;
                                 /*fprintf(stderr,"exchange_node->payload_info[%d][%d].r_offset %d\n",i,j,exchange_node->payload_info[i][j].r_offset);*/
-                                
+
                                 FIND_BASE(base_temp,reindex_myid,i,tree_order);
                                 ex_node = exchange_node->reindex_map[base_temp];
                                 knt2 = 0;
@@ -307,14 +317,14 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
                                 for(kk = 0; kk < myid; kk++){
                                     knt2 += hier_ranks[kk];
                                 }
-                                exchange_node->payload_info[i][j].s_offset = knt2; 
+                                exchange_node->payload_info[i][j].s_offset = knt2;
                                 /*fprintf(stderr,"exchange_node->payload_info[%d][%d].s_offset %d\n",i,j, exchange_node->payload_info[i][j].s_offset);*/
                             }
                             /* how much I am to receive from this peer on this level */
                             /* how much I am to send to this peer on this level */
                             exchange_node->payload_info[i][j].s_len = prev_data[node_rank];
                             /*fprintf(stderr,"exchange_node->payload_info[%d][%d].s_len %d\n",i,j,prev_data[node_rank]);*/
-                            /*fprintf(stderr,"I am rank %d receiveing %d bytes from rank %d at level %d\n",node_rank,
+                            /*fprintf(stderr,"I am rank %d receiving %d bytes from rank %d at level %d\n",node_rank,
                                                                         prev_data[exchange_node->reindex_map[peer]],
                                                                         exchange_node->reindex_map[peer], i+1);*/
                             /*fprintf(stderr,"I am rank %d sending %d bytes to rank %d at level %d\n",node_rank,prev_data[myid],
@@ -337,7 +347,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
             prev_data[j] = current_data[j];
         }
        /* fprintf(stderr,"\n");*/
-        
+
     }
 
 
@@ -360,7 +370,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
         exchange_node->n_extra_sources = 0;
         for( i = stray; i < num_nodes; i++) {
             if(exchange_node->reindex_myid == ( i - cnt )) {
-                /* then I am a proxy rank and there is only a 
+                /* then I am a proxy rank and there is only a
                  * single extra source
                  */
                 exchange_node->n_extra_sources = 1;
@@ -399,7 +409,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
         /* initialize this */
         exchange_node->n_actual_exchanges = 0;
         /* Allocate 2 dimension array thak keeps
-         rank exchange information for each step*/ 
+         rank exchange information for each step*/
         exchange_node->rank_exchanges = (int **) malloc
             (exchange_node->n_exchanges * sizeof(int *));
         if(NULL == exchange_node->rank_exchanges) {
@@ -426,12 +436,12 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
                 if ( peer < stray ) {
                     exchange_node->rank_exchanges[i][j] = exchange_node->reindex_map[peer];
                     /* an actual exchange occurs, bump the counter */
-                   
+
                 } else {
                     /* out of range, skip it - do not bump the n_actual_exchanges counter */
                     exchange_node->rank_exchanges[i][j] = -1;
                 }
-            
+
             }
             k_temp1 *= tree_order;
             k_temp2 *= tree_order;
@@ -457,7 +467,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_allgather_tree_node(
      */
     /* do we need this one */
     exchange_node->n_tags = tree_order * n_levels + 1;
-    
+
     free(prev_data);
     free(current_data);
     free(group_info);
@@ -496,22 +506,52 @@ Error:
     return OMPI_ERROR;
 }
 
+OMPI_DECLSPEC void ompi_netpatterns_cleanup_recursive_knomial_allgather_tree_node(
+        netpatterns_k_exchange_node_t *exchange_node)
+{
+    int i;
 
-OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_tree_node(
+    free(exchange_node->reindex_map);
+    free(exchange_node->inv_reindex_map);
+    if (exchange_node->n_extra_sources > 0) {
+        free(exchange_node->rank_extra_sources_array) ;
+        exchange_node->n_extra_sources = 0;
+        exchange_node->rank_extra_sources_array = NULL;
+    }
+    if (exchange_node->n_exchanges > 0) {
+        for (i=0; i < exchange_node->n_exchanges; i++) {
+            free(exchange_node->rank_exchanges[i]);
+            exchange_node->rank_exchanges[i] = NULL;
+        }
+        free(exchange_node->rank_exchanges);
+        exchange_node->rank_exchanges = NULL;
+        exchange_node->n_exchanges = 0;
+    }
+    for(i = 0; i < exchange_node->log_tree_order; i++){
+        free(exchange_node->payload_info[i]);
+    }
+    free(exchange_node->payload_info);
+}
+
+OMPI_DECLSPEC int ompi_netpatterns_setup_recursive_knomial_tree_node(
         int num_nodes, int node_rank, int tree_order,
         netpatterns_k_exchange_node_t *exchange_node)
 {
     /* local variables */
     int i, j, tmp, cnt;
     int n_levels;
-    int k_base, kpow_num, peer; 
+    int k_base, kpow_num, peer;
 
     NETPATTERNS_VERBOSE(
-            ("Enter netpatterns_setup_recursive_knomial_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)",
+            ("Enter ompi_netpatterns_setup_recursive_knomial_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)",
                 num_nodes, node_rank, tree_order));
 
     assert(num_nodes > 1);
     assert(tree_order > 1);
+    /* Also validate in release build where asserts are compiled out */
+    if ((num_nodes <= 1) || (tree_order <= 1)) {
+        goto Error;
+    }
     if (tree_order > num_nodes) {
         tree_order = num_nodes;
     }
@@ -590,7 +630,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_tree_node(
     if (EXCHANGE_NODE == exchange_node->node_type) {
         exchange_node->n_exchanges = n_levels;
         /* Allocate 2 dimension array thak keeps
-         rank exchange information for each step*/ 
+         rank exchange information for each step*/
         exchange_node->rank_exchanges = (int **) malloc
             (exchange_node->n_exchanges * sizeof(int *));
         if(NULL == exchange_node->rank_exchanges) {
@@ -604,18 +644,18 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_tree_node(
             }
         }
         /* fill in exchange partners */
-        for(i = 0, kpow_num = 1; i < exchange_node->n_exchanges; 
+        for(i = 0, kpow_num = 1; i < exchange_node->n_exchanges;
                                       i++, kpow_num *= tree_order) {
             k_base = node_rank / (kpow_num * tree_order);
             for(j = 1; j < tree_order; j++) {
                 peer = node_rank + kpow_num * j;
                 if (k_base != peer/(kpow_num * tree_order)) {
                     /* Wraparound the number */
-                    peer = k_base * (kpow_num * tree_order)  + 
+                    peer = k_base * (kpow_num * tree_order)  +
                         peer % (kpow_num * tree_order);
                 }
                 exchange_node->rank_exchanges[i][j - 1] = peer;
-                NETPATTERNS_VERBOSE(("rank_exchanges#(%d,%d)/%d = %d", 
+                NETPATTERNS_VERBOSE(("rank_exchanges#(%d,%d)/%d = %d",
                             i, j, tree_order, peer));
             }
         }
@@ -635,25 +675,35 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_knomial_tree_node(
 
 Error:
 
-    if (NULL != exchange_node->rank_extra_sources_array) {
-        free(exchange_node->rank_extra_sources_array);
-    }
-
-    if (NULL != exchange_node->rank_exchanges) {
-        for (i = 0; i < exchange_node->n_exchanges; i++) {
-            if (NULL != exchange_node->rank_exchanges[i]) {
-                free(exchange_node->rank_exchanges[i]);
-            }
-        }
-        free(exchange_node->rank_exchanges);
-    }
+    ompi_netpatterns_cleanup_recursive_knomial_tree_node (exchange_node);
 
     /* error return */
     return OMPI_ERROR;
 }
 
-#if 1 
-OMPI_DECLSPEC int netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes, int node_rank, int tree_order,
+OMPI_DECLSPEC void ompi_netpatterns_cleanup_recursive_knomial_tree_node(
+        netpatterns_k_exchange_node_t *exchange_node)
+{
+    int i;
+
+    if (exchange_node->n_extra_sources > 0) {
+        free(exchange_node->rank_extra_sources_array);
+        exchange_node->rank_extra_sources_array = NULL;
+        exchange_node->n_extra_sources = 0;
+    }
+    if (exchange_node->n_exchanges > 0) {
+        for (i=0 ; i<exchange_node->n_exchanges; i++) {
+            free(exchange_node->rank_exchanges[i]);
+            exchange_node->rank_exchanges[i] = NULL;
+        }
+        free(exchange_node->rank_exchanges);
+        exchange_node->rank_exchanges = NULL;
+        exchange_node->n_exchanges = 0;
+    }
+}
+
+#if 1
+OMPI_DECLSPEC int ompi_netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes, int node_rank, int tree_order,
         netpatterns_pair_exchange_node_t *exchange_node)
 {
     /* local variables */
@@ -661,7 +711,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes
     int n_levels;
     int shift, mask;
 
-    NETPATTERNS_VERBOSE(("Enter netpatterns_setup_recursive_doubling_n_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)", num_nodes, node_rank, tree_order));
+    NETPATTERNS_VERBOSE(("Enter ompi_netpatterns_setup_recursive_doubling_n_tree_node(num_nodes=%d, node_rank=%d, tree_order=%d)", num_nodes, node_rank, tree_order));
 
     assert(num_nodes > 1);
     while (tree_order > num_nodes) {
@@ -745,7 +795,7 @@ OMPI_DECLSPEC int netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes
         NETPATTERNS_VERBOSE(("extra_source#%d = %d", 0, node_rank & (cnt - 1)));
     }
 
-    /* Ishai: To be compatable with the old structure - should be remoived later */
+    /* Ishai: To be compatible with the old structure - should be removed later */
     if (1 == exchange_node->n_extra_sources) {
         exchange_node->rank_extra_source = exchange_node->rank_extra_sources_array[0];
     } else {
@@ -794,7 +844,7 @@ Error:
     return OMPI_ERROR;
 }
 
-OMPI_DECLSPEC void netpatterns_free_recursive_doubling_tree_node(
+OMPI_DECLSPEC void ompi_netpatterns_cleanup_recursive_doubling_tree_node(
     netpatterns_pair_exchange_node_t *exchange_node)
 {
     NETPATTERNS_VERBOSE(("About to release rank_extra_sources_array and rank_exchanges"));
@@ -808,15 +858,15 @@ OMPI_DECLSPEC void netpatterns_free_recursive_doubling_tree_node(
 }
 #endif
 
-OMPI_DECLSPEC int netpatterns_setup_recursive_doubling_tree_node(int num_nodes, int node_rank,
+OMPI_DECLSPEC int ompi_netpatterns_setup_recursive_doubling_tree_node(int num_nodes, int node_rank,
         netpatterns_pair_exchange_node_t *exchange_node)
 {
-    return netpatterns_setup_recursive_doubling_n_tree_node(num_nodes, node_rank, 2, exchange_node);
+    return ompi_netpatterns_setup_recursive_doubling_n_tree_node(num_nodes, node_rank, 2, exchange_node);
 }
 
-#if 0 
+#if 0
 /*OMPI_DECLSPEC int old_netpatterns_setup_recursive_doubling_tree_node(int num_nodes, int node_rank,*/
-OMPI_DECLSPEC int netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes, int node_rank,int tree_order,
+OMPI_DECLSPEC int ompi_netpatterns_setup_recursive_doubling_n_tree_node(int num_nodes, int node_rank,int tree_order,
         netpatterns_pair_exchange_node_t *exchange_node)
 {
     /* local variables */
